/* CRS (Channelflow with options of Rotation and Stratification), 
   revised Channelflow-1.1.2 (John F. Gibson) code including rotation and stratification
   Copyright (C) 2007-2008 Hao Lu

   Engine Research Center
   1500 Engineering Dr. #1011
   Madison, WI 53706

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CHANNELFLOW_TAUSOLVER_BC_H
#define CHANNELFLOW_TAUSOLVER_BC_H

#include "mathdefs.h"
#include "helmholtz.h"
#include "vector.h"
#include "chebyshev.h"

namespace channelflow {

class TauSolver_BC {
public:
  TauSolver_BC();  
  TauSolver_BC(BoundaryC BC, int kx, int kz, Real Lx, Real Lz, 
	Real a, Real b, Real lambda, Real nu, int nChebyModes);

  void solve(ChebyCoeff& s, const ChebyCoeff& Rs, Real sa, Real sb) const;

  void influenceCorrection(ChebyCoeff& s) const;
  
private:
  BoundaryC BC_; // boundary condtion
  int N_;        // number of Chebyshev modes
  int Nb_;       // N-1. Convenience variable, corresponds to C&H's N.
  int kx_;       // x wave number
  int kz_;       // z wave number
  Real two_pi_kxLx_; // 2 pi kx/Lx
  Real two_pi_kzLz_; // 2 pi kz/Lz
  Real kappa2_;  // 4 pi^2 [(kx/Lx)^2 + (kz/Lz)^2]
  Real a_;
  Real b_;
  Real lambda_;  
  Real nu_;      // viscosity
  
  HelmholtzSolver A_Helmholtz_;
  HelmholtzSolver B_Helmholtz_;
  
  ChebyCoeff sB_;

  Real DsB_; 
};

} //namespace channelflow
#endif
